package com.idwise.idwise_flutter

import android.app.Activity
import android.util.Log
import androidx.annotation.NonNull
import com.idwise.sdk.IDWise
import com.idwise.sdk.IDWiseSDKCallback
import com.idwise.sdk.data.models.IDWiseSDKError
import com.idwise.sdk.data.models.IDWiseSDKTheme
import com.idwise.sdk.data.models.JourneyInfo
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import org.json.JSONObject
import java.lang.reflect.Type
import com.idwise.sdk.IDWiseSDKStepCallback
import android.graphics.Bitmap
import android.util.Base64
import com.idwise.sdk.data.models.StepResult
import java.io.ByteArrayOutputStream


/** IdwiseFlutterPlugin */
class IdwiseFlutterPlugin : FlutterPlugin, MethodChannel.MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var activity: Activity
    private val TAG = "IdwiseFlutterPlugin"

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "idwise_flutter")
        channel.setMethodCallHandler(this)
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        onAttachedToActivity(binding)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
        // TODO: the Activity your plugin was attached to was
        // destroyed to change configuration.
        // This call will be followed by onReattachedToActivityForConfigChanges().
    }

    override fun onDetachedFromActivity() {
        // TODO: your plugin is no longer associated with an Activity.
        // Clean up references.
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        when (call.method) {
            "initialize" -> {
                IDWise.unloadSDK()

                val clientKey = call.argument<String>("clientKey")
                val theme = when (call.argument<String>("theme")) {
                    "LIGHT" -> IDWiseSDKTheme.LIGHT
                    "DARK" -> IDWiseSDKTheme.DARK
                    else -> IDWiseSDKTheme.SYSTEM_DEFAULT
                }

                IDWise.initialize(clientKey!!, theme) { error ->
                    Log.d("IDWiseSDKCallback", "onError ${error?.message}")
                    channel?.invokeMethod("onInitializeError", Gson().toJson(error))
                }
            }

            "startJourney" -> {
                val journeyDefinitionId = call.argument<String>("journeyDefinitionId")
                val referenceNo = call.argument<String>("referenceNo")
                val locale = call.argument<String>("locale")

                IDWise.startJourney(
                    activity,
                    journeyDefinitionId!!,
                    referenceNo,
                    locale,
                    callback = journeyCallback
                )
            }

            "startStep" -> {
                val stepId = call.argument<String>("stepId")
                stepId?.let { IDWise.startStep(activity, it); }
            }

            "startDynamicJourney" -> {
                Log.d("startDynamicJourney", "startDynamicJourney")
                val journeyDefinitionId = call.argument<String>("journeyDefinitionId")
                Log.d("startDynamicJourney", "journeyDefinitionId: $journeyDefinitionId")
                val referenceNo = call.argument<String>("referenceNo")
                Log.d("startDynamicJourney", "referenceNo: $referenceNo")
                val locale = call.argument<String>("locale")
                Log.d("startDynamicJourney", "locale: $locale")

                IDWise.startDynamicJourney(
                    activity,
                    journeyDefinitionId!!,
                    referenceNo,
                    locale,
                    journeyCallback,
                    stepCallback
                )
            }

            "resumeDynamicJourney" -> {
                Log.d("resumeDynamicJourney", "resumeDynamicJourney")
                val journeyDefinitionId = call.argument<String>("journeyDefinitionId")
                Log.d("resumeDynamicJourney", "journeyDefinitionId: $journeyDefinitionId")
                val journeyId = call.argument<String>("journeyId")
                Log.d("resumeDynamicJourney", "journeyId: $journeyId")
                val locale = call.argument<String>("locale")
                Log.d("resumeDynamicJourney", "locale: $locale")

                IDWise.resumeDynamicJourney(
                    activity,
                    journeyDefinitionId!!,
                    journeyId!!,
                    locale,
                    journeyCallback = journeyCallback,
                    stepCallback = stepCallback
                )
            }

            "finishDynamicJourney" -> {
                val journeyId = call.argument<String>("journeyId")
                journeyId?.let { IDWise.finishDynamicJourney(it) }
            }

            "getJourneySummary" -> {
                val journeyId = call.argument<String>("journeyId")
                journeyId?.let {
                    IDWise.getJourneySummary(it, callback = { summary, error ->

                        val gson = Gson()
                        val type: Type = object : TypeToken<HashMap<String, Any>>() {}.type
                        val argsMap = hashMapOf<String, Any?>()

                        summary?.let {
                            argsMap["summary"] = gson.fromJson(gson.toJson(summary), type)
                        }
                        error?.let {
                            argsMap["error"] = gson.fromJson(gson.toJson(error), type)
                        }

                        channel?.invokeMethod("journeySummary", argsMap)
                    })
                }
            }

            "unloadSDK" -> {
                IDWise.unloadSDK()
            }

            else -> result.error("NO_SUCH_METHOD", "NO SUCH METHOD", null)
        }

    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel?.setMethodCallHandler(null)
    }

    private val journeyCallback = object : IDWiseSDKCallback {
        override fun onJourneyStarted(journeyInfo: JourneyInfo) {
            Log.d("IDWiseSDKCallback", "onJourneyStarted")
            val argsMap = hashMapOf<String,String>().apply { 
                put("journeyId", journeyInfo.journeyId)
            }
            channel?.invokeMethod(
                "onJourneyStarted",
                argsMap
            )
        }

        override fun onJourneyCompleted(
            journeyInfo: JourneyInfo,
            isSucceeded: Boolean
        ) {
            Log.d("IDWiseSDKCallback", "onJourneyFinished")
            val argsMap = hashMapOf<String,String>().apply { 
                put("journeyId", journeyInfo.journeyId)
            }
            channel?.invokeMethod("onJourneyFinished", argsMap)
        }

        override fun onJourneyCancelled(journeyInfo: JourneyInfo?) {
            Log.d("IDWiseSDKCallback", "onJourneyCancelled")
            val argsMap = hashMapOf<String,String>().apply { 
                put("journeyId", journeyInfo?.journeyId?:"")
            }
            channel?.invokeMethod("onJourneyCancelled", argsMap)
        }

        override fun onJourneyResumed(journeyInfo: JourneyInfo) {
            Log.d("IDWiseSDKCallback", "onJourneyResumed")
            val argsMap = hashMapOf<String,String>().apply { 
                put("journeyId", journeyInfo.journeyId)
            }
            channel?.invokeMethod(
                "onJourneyResumed",
                argsMap
            )
        }

        override fun onError(error: IDWiseSDKError) {
            Log.d(
                "IDWiseSDKCallback",
                "onError ${error.message}"
            )
            val argsMap = hashMapOf<String, Any?>()
            argsMap["errorCode"] = error.errorCode
            argsMap["message"] = error.message
            
            channel?.invokeMethod("onError", argsMap)
        }
    }

    private val stepCallback = object : IDWiseSDKStepCallback {
        override fun onStepCaptured(
            stepId: String,
            bitmap: Bitmap?,
            croppedBitmap: Bitmap?
        ) {
            Log.d("IDWiseStepSDKCallback", "onStepCaptured")
            val bitmapString = convertImageToBase64String(croppedBitmap)

            val map = hashMapOf<String,String>().apply{
                put("stepId",stepId)
                put("capturedImage",bitmapString)
            }

            channel?.invokeMethod("onStepCaptured", map)
        }

        override fun onStepConfirmed(stepId: String) {
            Log.d("IDWiseStepSDKCallback", "onStepConfirmed")
            channel?.invokeMethod("onStepConfirmed", stepId)
        }

        override fun onStepResult(stepId: String, stepResult: StepResult?) {

            val gson = Gson()
            val type: Type = object : TypeToken<HashMap<String, Any>>() {}.type
            val argsMap = hashMapOf<String, Any?>()

            argsMap["stepId"] = stepId

            stepResult?.let {
                argsMap["stepResult"] = gson.fromJson(gson.toJson(stepResult), type)
            }

            channel?.invokeMethod("onStepResult" ,argsMap)
        }

    }

    fun convertImageToBase64String(bitmap: Bitmap?): String {
        if (bitmap == null || bitmap.isRecycled) return ""

        val baos = ByteArrayOutputStream()
        bitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos)
        val b = baos.toByteArray()
        return Base64.encodeToString(b, Base64.NO_WRAP)
    }
}
