import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'idwise_flutter.dart';
import 'idwise_flutter_platform_interface.dart';

/// An implementation of [IdwiseFlutterPlatform] that uses method channels.
class MethodChannelIdwiseFlutter extends IdwiseFlutterPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('idwise_flutter');

  Function(String)? _onInitializeErrorCallback;
  Function(dynamic)? _onJourneySummaryCallback;
  IDWiseSDKJourneyCallbacks? journeyCallbacks;
  IDWiseSDKStepCallbacks? stepCallbacks;

  MethodChannelIdwiseFlutter() {
    methodChannel.setMethodCallHandler((handler) async {
      switch (handler.method) {
        case 'onJourneyStarted':
          journeyCallbacks?.onJourneyStarted(handler.arguments);
          break;
        case 'onJourneyCompleted':
          journeyCallbacks?.onJourneyCompleted(handler.arguments);
          break;
        case 'onJourneyCancelled':
          journeyCallbacks?.onJourneyCancelled(handler.arguments);
          break;
        case 'onJourneyResumed':
          journeyCallbacks?.onJourneyResumed(handler.arguments);
          break;
        case 'onError':
          journeyCallbacks?.onError(handler.arguments);
          break;
        case 'onInitializeError':
          _onInitializeErrorCallback?.call(handler.arguments);
          break;
        case 'onStepCaptured':
          stepCallbacks?.onStepCaptured(handler.arguments);
          break;
        case 'onStepResult':
          stepCallbacks?.onStepResult(handler.arguments);
          break;
        case 'journeySummary':
          _onJourneySummaryCallback?.call(handler.arguments);
          break;
        default:
          print('Unknown method from MethodChannel: ${handler.method}');
          break;
      }
    });
  }

  @override
  Future<String?> getPlatformVersion() async {
    final version =
        await methodChannel.invokeMethod<String>('getPlatformVersion');
    return version;
  }

  Future<dynamic>? methodChannelHandler = Future(() => null);

  @override
  void initialize(String clientKey, IDWiseSDKTheme theme,
      {required onError}) async {
    print('initializing SDK with clientKey: $clientKey');
    _onInitializeErrorCallback = onError;

    methodChannel.invokeMethod(
        'initialize', {"clientKey": clientKey, "theme": theme.name});
  }

  @override
  void startJourney(String journeyDefinitionId, String referenceNo,
      String locale, IDWiseSDKJourneyCallbacks journeyCallbacks) {
    this.journeyCallbacks = journeyCallbacks;

    methodChannel.invokeMethod('startJourney', {
      "journeyDefinitionId": journeyDefinitionId,
      "referenceNo": referenceNo,
      "locale": locale
    });
  }

  @override
  void startDynamicJourney(
      String journeyDefinitionId,
      String referenceNo,
      String locale,
      IDWiseSDKJourneyCallbacks journeyCallbacks,
      IDWiseSDKStepCallbacks stepCallbacks) {
    this.journeyCallbacks = journeyCallbacks;
    this.stepCallbacks = stepCallbacks;

    methodChannel.invokeMethod('startDynamicJourney', {
      "journeyDefinitionId": journeyDefinitionId,
      "referenceNo": referenceNo,
      "locale": locale
    });
  }

  @override
  void resumeDynamicJourney(
      String journeyDefinitionId,
      String journeyId,
      String locale,
      IDWiseSDKJourneyCallbacks journeyCallbacks,
      IDWiseSDKStepCallbacks stepCallbacks) {
    this.journeyCallbacks = journeyCallbacks;
    this.stepCallbacks = stepCallbacks;
    methodChannel.invokeMethod('resumeDynamicJourney', {
      "journeyDefinitionId": journeyDefinitionId,
      "journeyId": journeyId,
      "locale": locale
    });
  }

  @override
  void getJourneySummary(String journeyId, {required journeySummary}) {
    _onJourneySummaryCallback = journeySummary;
    methodChannel.invokeMethod('getJourneySummary', {"journeyId": journeyId});
  }

  @override
  void startStep(String stepId) {
    methodChannel.invokeMethod('startStep', {"stepId": stepId});
  }

  @override
  void unloadSDK() {
    methodChannel.invokeMethod('unloadSDK', null);
  }
}
